
package de.codedo.conflicteditor.gui;

import de.codedo.conflicteditor.Difference;
import de.codedo.conflicteditor.gui.util.PreferencesAccess;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.StringWriter;
import java.util.List;
import java.util.Vector;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListModel;
import javax.swing.table.TableModel;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;

public class ConflictEditor
{
    private ConflictEditorController _controller;
    private ConflictEditorFrame _frame;

    public static void main(String[] args)
    {
        try
        {
            new ConflictEditor().run();
        }
        catch (Exception ex)
        {
            System.out.println("Uncaught toplevel Exception");
            ex.printStackTrace(System.out);
        }
    }

    private void run()
    {
        _controller = new ConflictEditorController();
        _frame = new ConflictEditorFrame();
        connectListeners();
        connectModels();
        _frame.pack();
        _frame.setVisible(true);
    }

    private void connectListeners()
    {
        connectDatabaseUrlCombo();
        connectFindConflictsButton();
        connectConflictsList();
        connectDifferencesTable();
        connectCurrentValueButton();
        connectConflictValueButton();
        connectPreviewButton();
        connectSaveButton();
    }

    private void connectDatabaseUrlCombo()
    {
        Executable<ActionEvent> executable = new Executable<ActionEvent>()
        {
            @Override
            public void execute(ActionEvent event) throws Exception
            {
                databaseUrlEdited(event);
            }
        };
        ExceptionHandlingActionListener listener = new ExceptionHandlingActionListener(executable);
        _frame.getDatabaseUrlCombo().addActionListener(listener);
    }

    private void connectFindConflictsButton()
    {
        Executable<ActionEvent> executable = new Executable<ActionEvent>()
        {
            @Override
            public void execute(ActionEvent event) throws Exception
            {
                findConflicts();
            }
        };
        ExceptionHandlingActionListener listener = new ExceptionHandlingActionListener(executable);
        _frame.getFindConflictsButton().addActionListener(listener);
    }

    private void connectConflictsList()
    {
        AdjustingSelectionExecutable executable = new AdjustingSelectionExecutable()
        {
            @Override
            protected void execute(int index) throws Exception
            {
                selectConflictDocument();
            }
        };
        ExceptionHandlingListSelectionListener listener = new ExceptionHandlingListSelectionListener(
            executable);
        _frame.getConflictsList().addListSelectionListener(listener);
    }

    private void connectDifferencesTable()
    {
        AdjustingSelectionExecutable executable = new AdjustingSelectionExecutable()
        {
            @Override
            protected void execute(int index) throws Exception
            {
                selectDifference(index);
            }
        };
        ExceptionHandlingListSelectionListener listener = new ExceptionHandlingListSelectionListener(
            executable);
        _frame.getDifferencesTable().getSelectionModel().addListSelectionListener(listener);
    }

    private void connectCurrentValueButton()
    {
        Executable<ActionEvent> executable = new Executable<ActionEvent>()
        {
            @Override
            public void execute(ActionEvent event) throws Exception
            {
                useCurrentValue();
            }
        };
        ExceptionHandlingActionListener listener = new ExceptionHandlingActionListener(executable);
        _frame.getCurrentValueButton().addActionListener(listener);
    }

    private void connectConflictValueButton()
    {
        Executable<ActionEvent> executable = new Executable<ActionEvent>()
        {
            @Override
            public void execute(ActionEvent event) throws Exception
            {
                useConflictValue();
            }
        };
        ExceptionHandlingActionListener listener = new ExceptionHandlingActionListener(executable);
        _frame.getConflictValueButton().addActionListener(listener);
    }

    private void connectPreviewButton()
    {
        Executable<ActionEvent> executable = new Executable<ActionEvent>()
        {
            @Override
            public void execute(ActionEvent event) throws Exception
            {
                preview();
            }
        };
        ExceptionHandlingActionListener listener = new ExceptionHandlingActionListener(executable);
        _frame.getPreviewButton().addActionListener(listener);
    }

    private void connectSaveButton()
    {
        Executable<ActionEvent> executable = new Executable<ActionEvent>()
        {
            @Override
            public void execute(ActionEvent event) throws Exception
            {
                save();
            }
        };
        ExceptionHandlingActionListener listener = new ExceptionHandlingActionListener(executable);
        _frame.getSaveButton().addActionListener(listener);
    }

    private void connectModels()
    {
        createAndSetDatabaseUrlComboModel();
    }

    private void createAndSetDatabaseUrlComboModel()
    {
        Vector<String> databaseUrls = new PreferencesAccess().getDatabaseUrls();
        ComboBoxModel model = new DefaultComboBoxModel(databaseUrls);
        _frame.getDatabaseUrlCombo().setModel(model);
    }

    protected void databaseUrlEdited(ActionEvent event)
    {
        if ("comboBoxEdited".equals(event.getActionCommand()))
        {
            databaseUrlEdited();
        }
    }

    private void databaseUrlEdited()
    {
        String dbUrl = _frame.getDatabaseUrl();
        new PreferencesAccess().storeDatabaseUrl(dbUrl);
    }

    protected void findConflicts() throws IOException
    {
        String dbUrl = _frame.getDatabaseUrl();
        JsonNode conflicts = _controller.findConflicts(dbUrl);
        ListModel model = new ConflictsListModel(conflicts);
        _frame.getConflictsList().setModel(model);
    }

    protected void selectConflictDocument() throws Exception
    {
        int selectedIndex = _frame.getConflictsList().getSelectedIndex();
        if (selectedIndex > -1)
        {
            selectConflictDocument(selectedIndex);
        }
        else
        {
            deselectConflictDocument();
        }
    }

    private void selectConflictDocument(int index) throws Exception
    {
        JsonNode conflictNode = (JsonNode)_frame.getConflictsList().getModel().getElementAt(index);
        _controller.setCurrentConflict(conflictNode);

        List<Difference> differences = _controller.differencesForConflict(conflictNode);
        TableModel model = new DifferenceTableModel(differences);
        _frame.getDifferencesTable().setModel(model);

        _frame.disableButtons();
    }

    private void deselectConflictDocument()
    {
        TableModel model = new DifferenceTableModel();
        _frame.getDifferencesTable().setModel(model);
    }

    protected void selectDifference(int index)
    {
        // the index that's passed in from the list selection event is wrong!
        index = _frame.getDifferencesTable().getSelectedRow();
        if (index > -1)
        {
            DifferenceTableModel model = (DifferenceTableModel)_frame.getDifferencesTable().getModel();
            Difference difference = model.differenceAtIndex(index);
            _controller.setCurrentDifference(difference);

            _frame.enableButtons();
        }
    }

    protected void useCurrentValue()
    {
        _controller.useCurrentValue();
    }

    protected void useConflictValue()
    {
        _controller.useConflictValue();
    }

    protected void preview() throws Exception
    {
        JsonNode currentConflict = _controller.getCurrentConflict();
        StringWriter writer = new StringWriter();
        new ObjectMapper().defaultPrettyPrintingWriter().writeValue(writer, currentConflict);

        JTextArea text = new JTextArea();
        text.setText(writer.toString());

        JScrollPane scrollPane = new JScrollPane(text);

        // TODO modal
        JFrame preview = new JFrame();
        preview.setPreferredSize(new Dimension(800, 600));
        preview.add(scrollPane);

        preview.pack();
        preview.setVisible(true);
    }

    protected void save() throws Exception
    {
        _controller.save();

        JsonNode resolvedConflict = _controller.getCurrentConflict();
        ConflictsListModel conflictsModel = (ConflictsListModel)_frame.getConflictsList().getModel();
        conflictsModel.remove(resolvedConflict);

        clearSelectionInConflictsList();
    }

    private void clearSelectionInConflictsList()
    {
        _frame.getConflictsList().getSelectionModel().clearSelection();
    }
}
